import torch
from ppo import PPO, Memory
from environments import MetaGames
import argparse


parser = argparse.ArgumentParser()
parser.add_argument("--game", type=str, required=True)
parser.add_argument("--opponent", type=str, required=True)
args = parser.parse_args()


if __name__ == "__main__":
    ############################################
    K_epochs = 4  # update policy for K epochs

    eps_clip = 0.2  # clip parameter for PPO
    gamma = 0.99  # discount factor

    lr = 0.002  # parameters for Adam optimizer
    betas = (0.9, 0.999)
    entropy = 0.01

    max_episodes = 1024
    batch_size = 4096
    random_seed = None
    num_steps = 100
    #############################################

    # creating environment
    env = MetaGames(batch_size, opponent=args.opponent, game=args.game)

    action_dim = env.d
    state_dim = env.d * 2

    memory = Memory()
    ppo = PPO(state_dim, action_dim, lr, betas, gamma, K_epochs, eps_clip, entropy)

    # logging variables
    running_reward = 0

    # training loop
    for i_episode in range(1, max_episodes + 1):
        state = env.reset()

        running_reward = torch.zeros(batch_size).cuda()
        running_opp_reward = torch.zeros(batch_size).cuda()

        last_reward = 0

        for t in range(num_steps):

            # Running policy_old:
            action = ppo.policy_old.act(state, memory)
            state, reward, info, M = env.step(action)

            memory.rewards.append(reward)
            running_reward += reward.squeeze(-1)
            running_opp_reward += info.squeeze(-1)
            last_reward = reward.squeeze(-1)

        ppo.update(memory)
        memory.clear_memory()

        print("=" * 100, flush=True)

        print(f"episode: {i_episode}", flush=True)

        print(f"loss: {-running_reward.mean() / num_steps}", flush=True)

        print(f"opponent loss: {-running_opp_reward.mean() / num_steps}", flush=True)
